@@ -13,13 +13,13 @@ import logging
 import platform
 from time import sleep
 
-_TMP_ELF_FILE = "/tmp/program.elf"
 _LOGGER = logging.getLogger(__name__)
+
 class ConfigException(Exception):
     def __init__(self, message):
         self.message = message
 
-def openocd_program(cmd):
+def openocd_cmd(cmd):
     killall("openocd")
     proc = subprocess.Popen(cmd, stderr=subprocess.PIPE, universal_newlines=True, cwd=os.getcwd(), shell=True)
     for ln in iter(proc.stderr.readline, ""):
@@ -29,14 +29,13 @@ def openocd_program(cmd):
     if ret:
         _LOGGER.error("openocd_program failed: '{}', returncode: {}".format(cmd, ret))
         raise subprocess.CalledProcessError(ret, cmd)
-    os.unlink(_TMP_ELF_FILE)
 
-def openocd_reset_device(cmd):
-    killall("openocd")
-    proc = subprocess.run(cmd, cwd=os.getcwd(), shell=True)
-    if proc.returncode != 0:
-        _LOGGER.error("openocd_reset_device failed: '{}', returncode: {}".format(cmd, proc.returncode))
-        raise subprocess.CalledProcessError(proc.returncode, cmd)
+#def openocd_cmd(cmd):
+#    killall("openocd")
+#    proc = subprocess.run(cmd, cwd=os.getcwd(), shell=True)
+#    if proc.returncode != 0:
+#        _LOGGER.error("openocd_cmd failed: '{}', returncode: {}".format(cmd, proc.returncode))
+#        raise subprocess.CalledProcessError(proc.returncode, cmd)
 
 def openocd_start_debug(cmd):
     killall("openocd")
@@ -47,20 +46,24 @@ def openocd_start_debug(cmd):
         _LOGGER.error("openocd_start_debug failed: '{}', returncode: {}".format(cmd, ret))
         raise subprocess.CalledProcessError(ret, 'openocd_start_debug')
 
+
 def openocd_terminate():
     killall("openocd")
 
+
 def killall(name):
     l = process_pid_list(name)
     for pid in l:
         os.kill(pid, signal.SIGTERM)
 
-def write_file(request_iterator):
-        file = open(_TMP_ELF_FILE, "wb")
+
+def write_stream_to_file(filename, request_iterator):
+        file = open(filename, "wb")
         for request in request_iterator:
             file.write(request.data)
         file.close()
 
+
 def process_pid_list(name):
     res = []
     for proc in psutil.process_iter(['pid', 'name']):
